#include <math.h>
#include "perf.h"
#include "memory.h"
#include "params.h"

// la macro PERF(n,ops,bytes) est utilisee dans chaque fonction
// dont on souhaite mesurer la performance :
// n*ops operations (calcul) et n*bytes octets sont comptabilises
// a chaque appel pour calculer la performance en termes de calcul
// (Gop/s) et de debit memoire (Gb/s)

/**************** Conditions aux limites periodiques ******************/

// p(y,x) = p[N*i+j]   y=i*dx, x=j*dx

static void copy(array out, array_in in, int n, int stride)
{ 
  for(int i=0; i<n; i++,in+=stride,out+=stride) *out=*in;
}

void periodize(array p, array_in q)
{
  copy(p,         q+N*(M-2), N,1); // p[0][j]  := q[M-2][j]   j=0..N-1
  copy(p+N*(M-1), q+N,       N,1); // p[M-1][j]:= q[1][j]     j=0..N-1
  copy(p,         q+N-2,     M,N); // p[i][0]  := q[i][N-2]   i=0..M-1
  copy(p+N-1,     q+1,       M,N); // p[i][N-1]:= q[i][1]     i=0..M-1
}

/*********************** Differences finies **************************/

static void diff(real * NOALIAS out, real alpha,
		 const real * NOALIAS in1,const real * NOALIAS in2)
{
  for(int i=0; i<N*M; i++)
    out[i]=alpha*(in2[i]-in1[i]);
}

void dpdx(array u, array_in p)
{
  PERF(M*N,2,3*sizeof(real));
  diff(u, 1./dx,p,p+N);
}
void dudx(array p, array_in u)
{
  PERF(M*N,2,3*sizeof(real));
  diff(p+N, 1./dx,u,u+N);
}
void dpdy(array v, array_in p)
{
  PERF(M*N,2,3*sizeof(real));
  diff(v, 1./dx,p,p+1);
}
void dvdy(array p, array_in v)
{
  PERF(M*N,2,3*sizeof(real));
  diff(p+1, 1./dx,v,v+1);
}

/******************************* Addition *****************************/

static void do_add(real * NOALIAS out, const real * NOALIAS in1, 
		real alpha,const real * NOALIAS in2) 
{
  for(int i=0; i<M*N; i++)
    out[i]= in1[i]+alpha*in2[i];
}
void add(array p, array_in q1, real alpha, array_in q2)
{
  PERF(M*N,2,3*sizeof(real));
  do_add(p,q1,alpha,q2);
}
