/********** faire "make CPPFLAGS=-R8" pour calculer en double precision *******/

#ifdef R8
typedef double real;
#else
typedef float real;
#endif

// le mot-cle __restrict__ aide g++ a optimiser                                                                               
#define NOALIAS __restrict__
typedef real * NOALIAS array;
typedef const real * NOALIAS array_in;

/********************** differents jeux de parametres *************************/
// le domain de calcul fait M*N poins (Mdx * Ndx)
// la simulation dure 2*P*Q pas de temps
// le champ de pression est sauvegarde tous les 2*Q pas de temps

// pour debugger, juste quelques pas de temps
enum {M=256, N=256, P=2, Q=1};

// haute resolution, simulation courte
// enum {M=1024, N=1024, P=4, Q=20};

// haute resolution, simulation longue
// enum {M=1024, N=1024, P=40, Q=40};

// basse resolution, simulation longue
// enum {M=256, N=256, P=200, Q=40};

/*********** vitesse de propagation c, pas d'espace dx, pas de temps dt *******/
// le pas de temps est calcule en fonction du pas d'espace
// pour respecter une contrainte de stabilite numerique
// (critere de Courant-Friedrichs-Lewy)
static const real c=1.0, f=1.0, dx=10./M, dt=.2*dx/c;

/****************** declarations des routines de finite_diff ******************/

void dpdx(array, array_in);
void dudx(array, array_in);
void dpdy(array, array_in);
void dvdy(array, array_in);
void periodize(array, array_in);
void add(array, array_in, real,array_in);

/*********** declarations des routines de saint_venant ***************/

void dynp(array, array_in,array_in,array_in);
void pvort_massflux(array,array,array, array_in,array_in,array_in);
void stepp(real,array, array_in,array_in,array_in);
void stepu(real,array, array_in,array_in,array_in,array_in);
void stepv(real,array, array_in,array_in,array_in,array_in);
