#include <vector>
#include <algorithm>
#include <assert.h>
#include <stdio.h>

#include "perf.h"

double perf::stack[16];
int perf::level=0;
perf *perf::list=NULL;

ostream& operator<<(ostream& out, perf info) { return info>>out;}

template<class type> static void padded_print(ostream& out, int n, const char *format, type data)
{
  static char space[2000];
  char *p=space;
  sprintf(space,format,data);
  while(*p && p<(space+n)) p++;
  while(p<(space+n)) *p++=' ';
  *p++=' '; *p=0;
  out<<space;
}

void perf::enter(double nops, double nbytes)
{
  if(!registered)
    {
      next=list;
      list=this;
      registered=-1;
      calls=0;
      total=child=0;
    }
  calls++; ops+=nops; bytes+=nbytes; 
  assert(level<15); stack[++level]=0;
}

ostream& perf::operator>>(ostream& out)
{
  double flops=0., nbytes=0.;
  if(total>0.)
    {
      flops  = ops/own/1e9; 
      nbytes = bytes/total/1e9;
    }
  padded_print(out,85,"%s",function);
  padded_print(out,5,"%f",total);
  padded_print(out,5,"%f",child);
  padded_print(out,5,"%f",own_percent);
  padded_print(out,5,"%d",calls);
  padded_print(out,5,"%f",1000*own/calls);
  padded_print(out,5,"%f",flops);
  padded_print(out,5,"%f",nbytes);
  return out;
}

void perf::display(double frac)
{
  int i=0,n=0;
  perf *p;
  double total=0., total_ops=0, total_bytes=0, cumul=0.;
  // count the number of profiled blocks
  for(p=list; p ; p=p->next) if(p->calls) n++;
  
  cout << '\n';
  padded_print(cout,30,"%s","Performance information for");
  padded_print(cout,3,"%d",n);
  padded_print(cout,50,"%s"," blocks :");
  padded_print(cout,5,"%s","total");
  padded_print(cout,5,"%s","child");
  padded_print(cout,3,"%s"," % ");
  padded_print(cout,5,"%s","calls");
  padded_print(cout,8,"%s"," ms/call");
  padded_print(cout,5,"%s","Gop/s");
  padded_print(cout,5,"%s","Gb/s");
  cout << '\n';
  
  vector<perf> v(n);
  // compute own_time ; insert data into a vector ; sort
  for(p=list; p ; p=p->next)
    {
      p->own = p->total - p->child;
      if(p->calls) 
	{ 
	  v[i++]=*p; 
	  total       += p->own;
	  total_ops   += p->ops;
	  total_bytes += p->bytes;
	}
    }
  sort(v.begin(),v.end());
  // print
  for(i=0;i<n;i++)
    {
      cumul+=v[i].own; v[i].own_percent=v[i].own/(total/100.);
      if(cumul<=frac*total) cout << v[i] << '\n';
    }
  cout << "Total time spent in profiled code (seconds) :" << total << '\n';
  cout << "Global performance : " << float(total_ops/total/1e9) << " Gop/s , " 
       << float(total_bytes/total/1e9) << " Gb/s\n";
}

void perf::reset()
{
  perf *p;
  for(p=list; p ; p=p->next) {p->reset_();}
}
