#include <math.h>
#include "perf.h"
#include "memory.h"
#include "params.h"

// equation des ondes : H=p, q=0, U=u, V=v

// stencil (-1,-N) <=> dvdy,dudx
void dynp(array H, array_in p,array_in u,array_in v)
{
  PERF(M*N,4,4*sizeof(real));
  for(int i=N; i<N*M; i++)
    {
      real uW=u[i-N],uE=u[i], vS=v[i-1],vN=v[i];
#ifdef WAVEQ
      H[i] = p[i];
#else
      H[i] = p[i]+.25*(uW*uW+uE*uE+vS*vS+vN*vN);
#endif
    }
}

// stencil (+1,+N) <=> dpdy, dpdx
void pvort_massflux(array q,array U,array V, array_in p,array_in u,array_in v)
{
  PERF(M*N,15,6*sizeof(real));
  real dx4 = .25*dx, fdx=f*dx;
  for(int i=0; i<N*(M-1); i++)
    {
      real pSW=p[i],pSE=p[i+N], pNW=p[i+1],pNE=p[i+N+1];
      real ppp = dx4*(pSW+pSE+pNW+pNE);     // 4 ops
#ifdef WAVEQ
      real c2=c*c;
      q[i] = f/c2;
      U[i] = c2*u[i];
      V[i] = c2*v[i];
#else
      q[i] = (fdx+v[i+N]-v[i]-u[i+1]+u[i])/ppp; // 5 ops
      U[i] = u[i]*.5*(pSW+pSE);             // 3 ops
      V[i] = v[i]*.5*(pSW+pNW);             // 3 ops
#endif
    }
}

// stencil (-1,-N) <=> dvdy, dudx
void stepp(real taudx,array p1, array_in p0, array_in U,array_in V)
{
  PERF(M*N,8,4*sizeof(real));
  taudx = taudx/dx;
  for(int i=N; i<N*(M-1); i++)
      p1[i] = p0[i] + taudx*(U[i-N]-U[i]+V[i-1]-V[i]);
}

// stencil (-1,+N) <=> dvdy, dpdx
void stepu(real tau,array u1, array_in u0, array_in q,array_in V,array_in H)
{
  PERF(M*N,11,6*sizeof(real));
  real kx=1./dx;
  for(int i=N; i<N*(M-1); i++)
    {
      real vm = V[i-1]+V[i]+V[i+N-1]+V[i+N];  // 3 ops
      real trend = kx*(H[i+N]-H[i]);          // 3 ops
      trend = trend - .125*vm*(q[i-1]+q[i]);  // 3 ops
      u1[i] = u0[i] - tau*trend;              // 2 ops
    }
}

// stencil (+1,-N) <=> dpdy, dudx
void stepv(real tau,array v1, array_in v0, array_in q,array_in U,array_in H)
{
  PERF(M*N,11,6*sizeof(real));
  real kx=1./dx;
  for(int i=N; i<N*(M-1); i++)
    {
      real um = U[i+1]+U[i]+U[i-N+1]+U[i-N]; // 3 ops 
      real trend = kx*(H[i+1]-H[i]);         // 3 ops
      trend = trend + .125*um*(q[i-N]+q[i]); // 3 ops
      v1[i] = v0[i] - tau*trend;             // 2 ops
    }
}
